-- ----------------------------
-- 会议记录表
-- ----------------------------
DROP TABLE IF EXISTS `mortals_xhx_metting_record`;
CREATE TABLE mortals_xhx_metting_record(
    `id` bigint(20) NOT NULL  AUTO_INCREMENT   COMMENT '主键ID，主键，自增长',
    `meetName` varchar(128) NOT NULL     COMMENT '会议名称',
    `meetDate` datetime NOT NULL     COMMENT '会议日期',
    `meetTimeStart` datetime NOT NULL     COMMENT '开会开始时间',
    `meetTimeEnd` datetime NOT NULL     COMMENT '开会结束时间',
    `roomId` bigint(20) NOT NULL     COMMENT '开会房间ID',
    `roomName` varchar(128) NOT NULL     COMMENT '开会房间名称',
    `meetDeptName` varchar(128) NOT NULL     COMMENT '开会部门名称',
    `meetStatus` tinyint(2) NOT NULL   DEFAULT '0'  COMMENT '会议状态(0.等待中,1.进行中,2.已结束)',
    `finishMethod` tinyint(2) NOT NULL   DEFAULT '0'  COMMENT '会议结束方式(0.自动结束,1.手动结束)',
    `remark` varchar(255)      COMMENT '备注',
    `createUserId` bigint(20) NOT NULL     COMMENT '创建用户',
    `createTime` datetime NOT NULL     COMMENT '创建时间',
    `updateUserId` bigint(20)      COMMENT '更新用户',
    `updateTime` datetime      COMMENT '更新时间',
PRIMARY KEY  (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='会议记录';
-- ----------------------------
-- 来往记录表
-- ----------------------------
DROP TABLE IF EXISTS `mortals_xhx_gocome_record`;
CREATE TABLE mortals_xhx_gocome_record(
    `id` bigint(20) NOT NULL  AUTO_INCREMENT   COMMENT '主键ID，主键，自增长',
    `userName` varchar(128) NOT NULL     COMMENT '用户名称',
    `phone` varchar(64)      COMMENT '电话',
    `deptName` varchar(64)      COMMENT '所属部门',
    `roomId` bigint(20) NOT NULL     COMMENT '所属房间ID',
    `roomName` varchar(128) NOT NULL     COMMENT '所属房间名称',
    `beforeStatus` varchar(64) NOT NULL     COMMENT '修改前状态',
    `afterStatus` varchar(64) NOT NULL     COMMENT '修改后状态',
    `remark` varchar(255)      COMMENT '备注',
    `createUserId` bigint(20) NOT NULL     COMMENT '创建用户',
    `createTime` datetime NOT NULL     COMMENT '创建时间',
    `updateUserId` bigint(20)      COMMENT '更新用户',
    `updateTime` datetime      COMMENT '更新时间',
PRIMARY KEY  (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='来往记录';
-- ----------------------------
-- 房间管理表
-- ----------------------------
DROP TABLE IF EXISTS `mortals_xhx_room`;
CREATE TABLE mortals_xhx_room(
    `id` bigint(20)   AUTO_INCREMENT   COMMENT '主键ID，主键，自增长',
    `roomName` varchar(64) NOT NULL     COMMENT '房间名称',
    `roomCode` varchar(10)      COMMENT '房间编码',
    `roomType` tinyint(2) NOT NULL   DEFAULT '0'  COMMENT '房间类型(0.会议室,1.办公室)',
    `deviceId` bigint(20)      COMMENT '绑定的电子门牌的设备ID',
    `deviceName` varchar(64)      COMMENT '绑定的设备名称',
    `countPerson` bigint(20)      COMMENT '员工人数',
    `remark` varchar(64)      COMMENT '备注',
    `createUserId` bigint(20) NOT NULL     COMMENT '创建用户',
    `createTime` datetime NOT NULL     COMMENT '创建时间',
    `updateUserId` bigint(20)      COMMENT '更新用户',
    `updateTime` datetime      COMMENT '更新时间',
    `deviceCode` varchar(64)      COMMENT '绑定的设备编码',
PRIMARY KEY  (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='房间管理';

-- ----------------------------
-- 房间工作人员信息表
-- ----------------------------
DROP TABLE IF EXISTS `mortals_xhx_room_workman`;
CREATE TABLE mortals_xhx_room_workman(
    `id` bigint(20)   AUTO_INCREMENT   COMMENT '主键ID，主键，自增长',
    `roomId` bigint(20)      COMMENT '房间id',
    `roomName` varchar(64) NOT NULL     COMMENT '房间名称',
    `workmanId` bigint(20)      COMMENT '工作人员id',
    `workmanName` varchar(64) NOT NULL     COMMENT '工作人员名称',
    `remark` varchar(64)      COMMENT '备注',
    `createUserId` bigint(20) NOT NULL     COMMENT '创建用户',
    `createTime` datetime NOT NULL     COMMENT '创建时间',
    `updateUserId` bigint(20)      COMMENT '更新用户',
    `updateTime` datetime      COMMENT '更新时间',
PRIMARY KEY  (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='房间工作人员信息';
-- ----------------------------
-- 工作人员表
-- ----------------------------
DROP TABLE IF EXISTS `mortals_xhx_workman`;
CREATE TABLE mortals_xhx_workman(
    `id` bigint(20)   AUTO_INCREMENT   COMMENT '序号，主键，自增长',
    `roomId` bigint(20)      COMMENT '所属房间id',
    `roomName` varchar(64)      COMMENT '所属房间名称',
    `deptName` varchar(64)      COMMENT '所属部门',
    `name` varchar(64)      COMMENT '姓名',
    `sex` tinyint(2)    DEFAULT '0'  COMMENT '性别(0.男,1.女)',
    `userPost` varchar(64)      COMMENT '职务',
    `phone` varchar(64)      COMMENT '电话',
    `mobile` varchar(64)      COMMENT '手机',
    `summary` varchar(255)      COMMENT '简介',
    `photoPath` varchar(255)      COMMENT '照片',
    `workStatus` varchar(64) NOT NULL     COMMENT '工作状态，来源基础设置',
    `createTime` datetime      COMMENT '创建时间',
    `createUserId` bigint(20)      COMMENT '创建用户',
    `updateTime` datetime      COMMENT '修改时间',
    `loginName` varchar(64)      COMMENT '登录用户名',
    `loginPwd` varchar(64)      COMMENT '密码',
PRIMARY KEY  (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='工作人员';

-- ----------------------------
-- 设备管理表
-- ----------------------------
DROP TABLE IF EXISTS `mortals_xhx_device`;
CREATE TABLE mortals_xhx_device(
    `id` bigint(20)   AUTO_INCREMENT   COMMENT '主键ID，主键，自增长',
    `deviceName` varchar(20) NOT NULL     COMMENT '设备名称',
    `roomId` bigint(20)      COMMENT '所属房间id',
    `roomName` varchar(64)      COMMENT '所属房间名称',
    `deviceCode` varchar(256)      COMMENT '设备编码,SN码等，默认为MAC地址',
    `deviceMac` varchar(64)      COMMENT '设备的MAC地址',
    `siteId` bigint(20)    DEFAULT '1'  COMMENT '站点Id，来源基础服务平台',
    `siteCode` varchar(256)      COMMENT '站点编号，来源基础服务平台',
    `siteName` varchar(256)      COMMENT '站点名称',
    `productCode` varchar(256)      COMMENT '产品编码',
    `productName` varchar(256)      COMMENT '产品名称',
    `deviceFirmname` varchar(20)      COMMENT '设备生产厂商名称',
    `resolution` tinyint(2)    DEFAULT '0'  COMMENT '分辨率(0.1920*1080,1.1366*768,2.1080*1920)',
    `deviceInBuilding` tinyint(2)      COMMENT '所属楼栋',
    `deviceInFloor` tinyint(2)      COMMENT '所属楼层',
    `defectsLiabilityPeriod` datetime      COMMENT '保修期至',
    `leadingOfficial` varchar(64)      COMMENT '负责人',
    `deviceAddr` varchar(64)      COMMENT '设备位置',
    `leadingOfficialTelephone` varchar(64)      COMMENT '联系电话',
    `devicePhotoPath` varchar(256)      COMMENT '设备图片',
    `ip` varchar(64)      COMMENT '设备访问ip',
    `centernum` varchar(64)      COMMENT '中心设备编码',
    `port` varchar(64)      COMMENT '端口',
    `deviceStatus` tinyint(2)    DEFAULT '0'  COMMENT '设备状态 (0.未激活，1.离线,2.在线)',
    `enabled` tinyint(2)    DEFAULT '1'  COMMENT '启用状态 (0.停止，1.启用)',
    `deviceRemark` varchar(256)      COMMENT '备注',
    `onlineTime` datetime      COMMENT '最近上线时间',
    `offlineTime` datetime      COMMENT '最近离线时间',
    `createUserId` bigint(20) NOT NULL     COMMENT '创建用户',
    `createTime` datetime NOT NULL     COMMENT '创建时间',
    `updateUserId` bigint(20)      COMMENT '更新用户',
    `updateTime` datetime      COMMENT '更新时间',
PRIMARY KEY  (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='设备管理';
-- ----------------------------
-- 状态设置表
-- ----------------------------
DROP TABLE IF EXISTS `mortals_xhx_basic_status`;
CREATE TABLE mortals_xhx_basic_status(
    `id` bigint(20)   AUTO_INCREMENT   COMMENT '主键ID，自增长',
    `basicSetId` bigint(20)      COMMENT '基础设置id',
    `statusName` varchar(64)      COMMENT '状态名称',
    `statusColor` varchar(64)      COMMENT '状态颜色值',
    `remark` varchar(64)      COMMENT '备注',
    `createUserId` bigint(20) NOT NULL     COMMENT '创建用户',
    `createTime` datetime NOT NULL     COMMENT '创建时间',
    `updateUserId` bigint(20)      COMMENT '更新用户',
    `updateTime` datetime      COMMENT '更新时间',
PRIMARY KEY  (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='状态设置';
-- ----------------------------
-- 基础设置表
-- ----------------------------
DROP TABLE IF EXISTS `mortals_xhx_basic_set`;
CREATE TABLE mortals_xhx_basic_set(
    `id` bigint(20)   AUTO_INCREMENT   COMMENT '主键ID，主键，自增长',
    `showPhoto` tinyint(1)    DEFAULT '0'  COMMENT '是否显示照片(0.否,1.是)',
    `showName` tinyint(1)    DEFAULT '0'  COMMENT '是否显示姓名 (0.否,1.是)',
    `showPost` tinyint(1)    DEFAULT '0'  COMMENT '是否显示职位(0.否,1.是)',
    `showWork` tinyint(1)    DEFAULT '0'  COMMENT '是否显示在岗状态(0.否,1.是)',
    `showPhone` tinyint(1)    DEFAULT '0'  COMMENT '是否显示电话(0.否,1.是)',
    `showElect` tinyint(1)    DEFAULT '0'  COMMENT '显示电子门牌信息(0.显示时间,1.自定义内容)',
    `showElectDesc` varchar(128)      COMMENT '显示电子门牌信息自定义内容',
    `showMeet` tinyint(1)    DEFAULT '0'  COMMENT '显示会议室门牌信息(0.显示时间,1.自定义内容)',
    `showMeetDesc` varchar(128)      COMMENT '显示会议室门牌信息自定义内容',
    `remark` varchar(64)      COMMENT '备注',
    `createUserId` bigint(20) NOT NULL     COMMENT '创建用户',
    `createTime` datetime NOT NULL     COMMENT '创建时间',
    `updateUserId` bigint(20)      COMMENT '更新用户',
    `updateTime` datetime      COMMENT '更新时间',
PRIMARY KEY  (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='基础设置';
